%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%
% Definitions of builtins of module System:
%
:- module(prim_system,
	  [prim_getCPUTime/1,prim_getElapsedTime/1,prim_getArgs/1,
	   prim_getEnviron/2,prim_setEnviron/3,prim_unsetEnviron/2,
	   prim_getHostname/1,prim_getPID/1,prim_getProgName/1,
	   prim_system/2,prim_exitWith/2,prim_sleep/2,
	   prim_evalTime/4,prim_evalSpace/4]).

:- use_module('../prologbasics').
:- use_module('../basics').
:- use_module(library(system)).

prim_getCPUTime(MS) :- getRunTime(MS).

prim_getElapsedTime(MS) :- getElapsedTime(MS).

prim_getArgs(StringArgs) :-
        (rtargs(Args) -> true ; getProgramArgs(Args)),
        map2(atom2String,Args,StringArgs).

prim_getEnviron(Var,Value) :-
	string2Atom(Var,AtomVar),
	(getEnv(AtomVar,AtomValue) -> atom2String(AtomValue,Value)
	                            ; Value = []). % empty string if undefined

prim_setEnviron(Val,Var,'Prelude.()') :-
	string2Atom(Var,AtomVar),
	string2Atom(Val,AtomVal),
	setEnv(AtomVar,AtomVal).

prim_unsetEnviron(Var,'Prelude.()') :-
	string2Atom(Var,AtomVar),
	unsetEnv(AtomVar).

prim_getHostname(String) :-
        getHostname(Name),
        atom2String(Name,String).

prim_getPID(Pid) :- currentPID(Pid).

prim_getProgName(String) :-
        user:currentModule(Name),
        atom2String(Name,String).

prim_system(S,Status) :-
	string2Atom(S,Cmd),
	shellCmd(Cmd,Status).

prim_exitWith(Code,_) :- halt(Code).

prim_sleep(S,'Prelude.()') :- sleep(S).

% evaluate the argument to normal form and print the evaluation time:
?- block prim_evalTime(?,?,-,?).
prim_evalTime(T,NF,E0,E) :-
	garbage_collect,
	getRunTime(RTime1),
	getElapsedTime(ETime1),
	statistics(garbage_collection,[GC1,_,_]),
	normalizeAndCheck(T,NF,E0,E1),
	getRunTime(RTime2),
	getElapsedTime(ETime2),
	statistics(garbage_collection,[GC2,_,_]),
	writeErr('EVALTIME: Runtime: '),
	RTime is RTime2-RTime1, writeErr(RTime),
	writeErr(' msec.'), nlErr,
	writeErr('EVALTIME: Elapsed time: '),
	ETime is ETime2-ETime1, writeErr(ETime),
	writeErr(' msec.'), nlErr,
	writeErr('EVALTIME: Number of garbage collections: '),
	GC is GC2-GC1, writeErr(GC), nlErr,
	garbageCollectorOn, E1=E.

% evaluate the argument to normal form (without garbage collection)
% and print the evaluation time and memory usage:
?- block prim_evalSpace(?,?,-,?).
prim_evalSpace(T,NF,E0,E) :-
	garbage_collect,
	garbageCollectorOff,
	getRunTime(RTime1),
	getElapsedTime(ETime1),
	statistics(garbage_collection,[GC1,_,_]),
	statistics(global_stack,[GSU1,_]),
	statistics(local_stack,[LSU1,_]),
	normalizeAndCheck(T,NF,E0,E1),
	getRunTime(RTime2),
	getElapsedTime(ETime2),
	statistics(garbage_collection,[GC2,_,_]),
	statistics(global_stack,[GSU2,_]),
	statistics(local_stack,[LSU2,_]),
	writeErr('EVALTIME: Runtime: '),
	RTime is RTime2-RTime1, writeErr(RTime),
	writeErr(' msec.'), nlErr,
	writeErr('EVALTIME: Elapsed time: '),
	ETime is ETime2-ETime1, writeErr(ETime),
	writeErr(' msec.'), nlErr,
	writeErr('EVALTIME: Number of garbage collections: '),
	GC is GC2-GC1, writeErr(GC), nlErr,
	writeErr('EVALTIME: Global stack usage: '),
	GS is GSU2-GSU1, writeErr(GS), nlErr,
	writeErr('EVALTIME: Local stack usage: '),
	LS is LSU2-LSU1, writeErr(LS), nlErr,
	garbageCollectorOn, E1=E.
